/*
	File:		TestExceptions.cp

	Contains:	Implementation of class TTestExceptions

	Copyright:	 1992-1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef __TESTEXCEPTIONS__
#include "TestExceptions.h"
#endif

/*******************************************************************************
** External Function
********************************************************************************/

extern void ThrowException(long);

/*******************************************************************************
** TTestExceptions routine
********************************************************************************/

TTestExceptions::TTestExceptions()
{}

TTestExceptions::~TTestExceptions()
{}

void TTestExceptions::InitTest(Boolean, Boolean, int, char**)
{}

void TTestExceptions::EndTest(Boolean, Boolean)
{}

void TTestExceptions::RunTestIteration(Boolean verbose, Boolean debug)
{
	DebugTest(debug, "About to Test Exceptions");
	if (verbose)
		Printf("# INFO: Testing CATCH\n");
		
	TRY
		ThrowException(-10);
	CATCH(-10)
		if (verbose)
			Printf("# INFO: Caught the Exception!\n");
	CATCH_ALL
		Printf("# ERROR: Exception caught in CATCH_ALL - #%d\n", ErrorCode());
	ENDTRY

	if (verbose)
		Printf("# INFO: Testing multiple CATCHs\n");
		
	TRY
		ThrowException(-20);
	CATCH(-10)
		Printf("# ERROR: Exception caught in CATCH(-10) - #%d\n", ErrorCode());
	CATCH(-20)
		if (verbose)
			Printf("# INFO: Caught the Exception!\n");
	CATCH(-30)
		Printf("# ERROR: Exception caught in CATCH(-30) - #%d\n", ErrorCode());
	CATCH_ALL
		Printf("# ERROR: Exception caught in CATCH_ALL - #%d\n", ErrorCode());
	ENDTRY

	if (verbose)
		Printf("# INFO: Testing CATCH_ALL\n");
		
	TRY
		ThrowException(-21);
	CATCH(-10)
		Printf("# ERROR: Exception caught in CATCH(-10) - #%d\n", ErrorCode());
	CATCH(-20)
		Printf("# ERROR: Exception caught in CATCH(-20) - #%d\n", ErrorCode());
	CATCH(-30)
		Printf("# ERROR: Exception caught in CATCH(-30) - #%d\n", ErrorCode());
	CATCH_ALL
		if (ErrorCode() == -21)
		{
			if (verbose)
				Printf("# INFO: Exception caugh in CATCH_ALL\n");
		}
		else
			Printf("# ERROR: Exception caught in CATCH_ALL - #%d\n", ErrorCode());
	ENDTRY

	if (verbose)
		Printf("# INFO: Testing FINALLY WITH NO EXCEPTION\n");
		
	Boolean flag = false;
	Volatile(flag);
	
	TRY
	CATCH(-10)
		flag = true;
		Printf("# ERROR: Exception caught in CATCH(-10) - #%d\n", ErrorCode());
	CATCH(-20)
		flag = true;
		Printf("# ERROR: Exception caught in CATCH(-20) - #%d\n", ErrorCode());
	CATCH(-30)
		flag = true;
		Printf("# ERROR: Exception caught in CATCH(-30) - #%d\n", ErrorCode());
	CATCH_ALL
		flag = true;
		Printf("# ERROR: Exception caught in CATCH_ALL - #%d\n", ErrorCode());
	FINALLY
		if (flag)
			Printf("# ERROR: Reached FINALLY code after exception code\n");
		else
			if (verbose)
			Printf("# INFO: Reached FINALLY code\n");
	ENDTRY
	
	if (verbose)
		Printf("# INFO: Testing FINALLY WITH  EXCEPTION\n");
		
	flag = false;
	
	TRY
		ThrowException(-10);
	CATCH(-10)
		flag = true;
		if (verbose)
			Printf("# INFO: Exception caught in CATCH\n");
	CATCH(-20)
		Printf("# ERROR: Exception caught in CATCH(-20) - #%d\n", ErrorCode());
	CATCH(-30)
		Printf("# ERROR: Exception caught in CATCH(-30) - #%d\n", ErrorCode());
	CATCH_ALL
		Printf("# ERROR: Exception caught in CATCH_ALL - #%d\n", ErrorCode());
	FINALLY
		if (!flag)
		{
			Printf("# ERROR: Reached FINALLY code after wrong exception or no exception\n");
		}
		else
			if (verbose)
				Printf("# INFO: Reached FINALLY code properly\n");
	ENDTRY
	
}
